@@ -131,8 +131,9 @@ void TaskView::contentsMousePressEvent ( QMouseEvent * e )
     int leftborder = treeStepSize() * ( task->depth() + ( rootIsDecorated() ? 1 : 0)) + itemMargin();
     if ((leftborder < e->x()) && (e->x() < 19 + leftborder ))
     {
-      if ( task->isComplete() ) task->setPercentComplete( 0, _storage );
-      else task->setPercentComplete( 100, _storage );
+      if ( e->button() == LeftButton )
+        if ( task->isComplete() ) task->setPercentComplete( 0, _storage );
+        else task->setPercentComplete( 100, _storage );
     }
     emit updateButtons();
   }
@@ -360,7 +361,7 @@ void TaskView::scheduleSave()
 Preferences* TaskView::preferences() { return _preferences; }
 
 QString TaskView::save()
-// This saves the not-running tasks.
+// This saves the tasks. If they do not yet have an endDate, their startDate is also not saved.
 {
   kdDebug(5970) << "Entering TaskView::save" << endl;
   QString err = _storage->save(this);
@@ -382,16 +383,21 @@ long TaskView::count()
 
 void TaskView::startTimerFor(Task* task, QDateTime startTime )
 {
-  if (task != 0 && activeTasks.findRef(task) == -1) {
-    _idleTimeDetector->startIdleDetection();
-    task->setRunning(true, _storage, startTime);
-    activeTasks.append(task);
-    emit updateButtons();
-    if ( activeTasks.count() == 1 )
+  kdDebug(5970) << "Entering TaskView::startTimerFor" << endl;
+  if (save()==QString())
+  {
+    if (task != 0 && activeTasks.findRef(task) == -1) 
+    {
+      _idleTimeDetector->startIdleDetection();
+      task->setRunning(true, _storage, startTime);
+      activeTasks.append(task);
+      emit updateButtons();
+      if ( activeTasks.count() == 1 )
         emit timersActive();
-
-    emit tasksChanged( activeTasks);
+      emit tasksChanged( activeTasks);
+    }
   }
+  else KMessageBox::error(0,i18n("Saving is impossible, so timing is useless. \nSaving problems may result from a full harddisk, a directory name instead of a file name, or stale locks. Check that your harddisk has enough space, that your calendar file exists and is a file and remove stale locks, typically from ~/.kde/share/apps/kabc/lock."));
 }
 
 void TaskView::clearActiveTasks()
@@ -416,7 +422,7 @@ void TaskView::stopAllTimersAt(QDateTime qdt)
 // stops all timers for the time qdt. This makes sense, if the idletimedetector detected
 // the last work has been done 50 minutes ago.
 {
-  kdDebug(5970) << "Entering TaskView::stopAllTimersAt" << endl;
+  kdDebug(5970) << "Entering TaskView::stopAllTimersAt " << qdt << endl;
   for ( unsigned int i = 0; i < activeTasks.count(); i++ )
   {
     activeTasks.at(i)->setRunning(false, _storage, qdt, qdt);
@@ -691,9 +697,14 @@ void TaskView::deleteTask(bool markingascomplete)
 }
 
 void TaskView::extractTime(int minutes)
+// This procedure subtracts ''minutes'' from the active task's time in the memory.
+// It is called by the idletimedetector class.
+// When the desktop has been idle for the past 20 minutes, the past 20 minutes have 
+// already been added to the task's time in order for the time to be displayed correctly.
+// That is why idletimedetector needs to subtract this time first.
 {
   kdDebug(5970) << "Entering extractTime" << endl;
-  addTimeToActiveTasks(-minutes);
+  addTimeToActiveTasks(-minutes,false); // subtract minutes, but do not store it
 }
 
 void TaskView::autoSaveChanged(bool on)
