@@ -16,7 +16,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *   Send comments and bug fixes to jcorey@fruity.ath.cx
  *
@@ -36,11 +36,19 @@
  *   - http://www.ietf.org/internet-drafts/draft-daboo-imap-annotatemore-05.txt
  *          IMAP ANNOTATEMORE draft - April 2004.
  *
+ *
  * Supported URLs:
- *   - imap://server/ - Prompt for user/pass, list all folders in home directory
- *   - imap://user:pass@server/ - Uses LOGIN to log in
- *   - imap://user;AUTH=method:pass@server/ - Uses AUTHENTICATE to log in
- *   - imap://server/folder/ - List messages in folder
+ * \verbatim
+imap://server/
+imap://user:pass@server/
+imap://user;AUTH=method:pass@server/
+imap://server/folder/
+ * \endverbatim
+ * These URLs cause the following actions (in order):
+ *   - Prompt for user/pass, list all folders in home directory
+ *   - Uses LOGIN to log in
+ *   - Uses AUTHENTICATE to log in
+ *   - List messages in folder
  *
  * @note API notes:
  *   Not receiving the required write access for a folder means
@@ -87,7 +95,6 @@ extern "C" {
 
 #define IMAP_PROTOCOL "imap"
 #define IMAP_SSL_PROTOCOL "imaps"
-#define IMAP_TIMEOUT_IN_SECONDS 60
 
 using namespace KIO;
 
@@ -97,20 +104,6 @@ extern "C"
   KDE_EXPORT int kdemain (int argc, char **argv);
 }
 
-#ifdef HAVE_LIBSASL2
-static sasl_callback_t callbacks[] = {
-    { SASL_CB_ECHOPROMPT, NULL, NULL },
-    { SASL_CB_NOECHOPROMPT, NULL, NULL },
-    { SASL_CB_GETREALM, NULL, NULL },
-    { SASL_CB_USER, NULL, NULL },
-    { SASL_CB_AUTHNAME, NULL, NULL },
-    { SASL_CB_PASS, NULL, NULL },
-    { SASL_CB_GETOPT, NULL, NULL },
-    { SASL_CB_CANON_USER, NULL, NULL },
-    { SASL_CB_LIST_END, NULL, NULL }
-};
-#endif
-
 int
 kdemain (int argc, char **argv)
 {
@@ -124,7 +117,7 @@ kdemain (int argc, char **argv)
   }
 
 #ifdef HAVE_LIBSASL2
-  if ( sasl_client_init( callbacks ) != SASL_OK ) {
+  if ( sasl_client_init( NULL ) != SASL_OK ) {
     fprintf(stderr, "SASL library initialization failed!\n");
     ::exit (-1);
   }
@@ -145,7 +138,7 @@ kdemain (int argc, char **argv)
 #ifdef HAVE_LIBSASL2
   sasl_done();
 #endif
-  
+
   return 0;
 }
 
@@ -172,7 +165,7 @@ IMAP4Protocol::IMAP4Protocol (const QCString & pool, const QCString & app, bool
         (isSSL ? IMAP_SSL_PROTOCOL : IMAP_PROTOCOL), pool,
               app, isSSL), imapParser (), mimeIO (), outputBuffer(outputCache)
 {
-  outputBufferIndex = 0L;
+  outputBufferIndex = 0;
   mySSL = isSSL;
   readBuffer[0] = 0x00;
   relayEnabled = false;
@@ -269,6 +262,7 @@ IMAP4Protocol::get (const KURL & _url)
     if (aEnum == ITYPE_BOX || aEnum == ITYPE_DIR_AND_BOX)
     {
       // write the digest header
+      cacheOutput = true;
       outputLine
         ("Content-Type: multipart/digest; boundary=\"IMAPDIGEST\"\r\n", 55);
       if (selectInfo.recentAvailable ())
@@ -301,6 +295,8 @@ IMAP4Protocol::get (const KURL & _url)
         }
       }
       outputLine ("\r\n", 2);
+      flushOutput(QString::null);
+      cacheOutput = false;
     }
 
     if (aEnum == ITYPE_MSG || (aEnum == ITYPE_ATTACH && !decodeContent))
@@ -308,9 +304,6 @@ IMAP4Protocol::get (const KURL & _url)
 
     if (aSequence != "0:0")
     {
-      if (getLastHandled())
-        getLastHandled()->clear();
-
       QString contentEncoding;
       if (aEnum == ITYPE_ATTACH && decodeContent)
       {
@@ -347,7 +340,7 @@ IMAP4Protocol::get (const KURL & _url)
         if (cache)
           lastone = cache->getHeader ();
 
-        if (!cmd->isComplete ())
+        if (cmd && !cmd->isComplete ())
         {
           if ((aUpper.find ("BODYSTRUCTURE") != -1)
                     || (aUpper.find ("FLAGS") != -1)
@@ -361,15 +354,15 @@ IMAP4Protocol::get (const KURL & _url)
               // write the mime header (default is here message/rfc822)
               outputLine ("--IMAPDIGEST\r\n", 14);
               cacheOutput = true;
-              if (cache->getUid () != 0)
+              if (cache && cache->getUid () != 0)
                 outputLineStr ("X-UID: " +
                                QString::number(cache->getUid ()) + "\r\n");
-              if (cache->getSize () != 0)
+              if (cache && cache->getSize () != 0)
                 outputLineStr ("X-Length: " +
                                QString::number(cache->getSize ()) + "\r\n");
-              if (!cache->getDate ().isEmpty())
+              if (cache && !cache->getDate ().isEmpty())
                 outputLineStr ("X-Date: " + cache->getDate () + "\r\n");
-              if (cache->getFlags () != 0)
+              if (cache && cache->getFlags () != 0)
                 outputLineStr ("X-Flags: " +
                                QString::number(cache->getFlags ()) + "\r\n");
             } else cacheOutput = true;
@@ -378,9 +371,9 @@ IMAP4Protocol::get (const KURL & _url)
             cacheOutput = false;
             flushOutput(contentEncoding);
           }
-        }
+        } // if not complete
       }
-      while (!cmd->isComplete ());
+      while (cmd && !cmd->isComplete ());
       if (aEnum == ITYPE_BOX || aEnum == ITYPE_DIR_AND_BOX)
       {
         // write the end boundary
@@ -427,14 +420,17 @@ IMAP4Protocol::listDir (const KURL & _url)
     QString listStr = myBox;
     imapCommand *cmd;
 
-    if (!listStr.isEmpty () && !listStr.endsWith(myDelimiter))
+    if (!listStr.isEmpty () && !listStr.endsWith(myDelimiter) &&
+        mySection != "FOLDERONLY")
       listStr += myDelimiter;
+
     if (mySection.isEmpty())
     {
       listStr += "%";
     } else if (mySection == "COMPLETE") {
       listStr += "*";
     }
+    kdDebug(7116) << "IMAP4Protocol::listDir - listStr=" << listStr << endl;
     cmd =
       doCommand (imapCommand::clientList ("", listStr,
             (myLType == "LSUB" || myLType == "LSUBNOCHECK")));
@@ -470,7 +466,7 @@ IMAP4Protocol::listDir (const KURL & _url)
             }
           }
           if (boxOk)
-            doListEntry (aURL, myBox, (*it));
+            doListEntry (aURL, myBox, (*it), (mySection != "FOLDERONLY"));
           else // this folder is dead
             kdDebug(7116) << "IMAP4Protocol::listDir - suppress " << (*it).name() << endl;
         }
@@ -481,7 +477,7 @@ IMAP4Protocol::listDir (const KURL & _url)
         for (QValueListIterator < imapList > it = listResponses.begin ();
             it != listResponses.end (); ++it)
         {
-          doListEntry (aURL, myBox, (*it));
+          doListEntry (aURL, myBox, (*it), (mySection != "FOLDERONLY"));
         }
       }
       entry.clear ();
@@ -621,7 +617,7 @@ IMAP4Protocol::listDir (const KURL & _url)
     selectInfo.setAlert( 0 );
   }
 
-  kdDebug(7116) << "IMAP4Protcol::listDir - Finishing listDir" << endl;
+  kdDebug(7116) << "IMAP4Protocol::listDir - Finishing listDir" << endl;
   finished ();
 }
 
@@ -652,11 +648,12 @@ IMAP4Protocol::parseRelay (const QByteArray & buffer)
   } else if (cacheOutput)
   {
     // collect data
-    outputBuffer.open(IO_WriteOnly);
+    if ( !outputBuffer.isOpen() ) {
+      outputBuffer.open(IO_WriteOnly);
+    }
     outputBuffer.at(outputBufferIndex);
     outputBuffer.writeBlock(buffer, buffer.size());
     outputBufferIndex += buffer.size();
-    outputBuffer.close();
   }
 }
 
@@ -676,6 +673,7 @@ bool IMAP4Protocol::parseRead(QByteArray & buffer, ulong len, ulong relay)
     ssize_t readLen = myRead(buf, QMIN(len - buffer.size(), sizeof(buf) - 1));
     if (readLen == 0)
     {
+      kdDebug(7116) << "parseRead: readLen == 0 - connection broken" << endl;
       error (ERR_CONNECTION_BROKEN, myHost);
       setState(ISTATE_CONNECT);
       closeConnection();
@@ -735,18 +733,19 @@ bool IMAP4Protocol::parseReadLine (QByteArray & buffer, ulong relay)
       }
 
       readBufferLen -= copyLen;
-      if (readBufferLen) 
+      if (readBufferLen)
         memmove(readBuffer, &readBuffer[copyLen], readBufferLen);
       if (buffer[buffer.size() - 1] == '\n') return TRUE;
     }
     if (!isConnectionValid())
     {
+      kdDebug(7116) << "parseReadLine - connection broken" << endl;
       error (ERR_CONNECTION_BROKEN, myHost);
       setState(ISTATE_CONNECT);
       closeConnection();
       return FALSE;
     }
-    if (!waitForResponse( IMAP_TIMEOUT_IN_SECONDS ))
+    if (!waitForResponse( responseTimeout() ))
     {
       error(ERR_SERVER_TIMEOUT, myHost);
       setState(ISTATE_CONNECT);
@@ -756,6 +755,7 @@ bool IMAP4Protocol::parseReadLine (QByteArray & buffer, ulong relay)
     readBufferLen = read(readBuffer, IMAP_BUFFER - 1);
     if (readBufferLen == 0)
     {
+      kdDebug(7116) << "parseReadLine: readBufferLen == 0 - connection broken" << endl;
       error (ERR_CONNECTION_BROKEN, myHost);
       setState(ISTATE_CONNECT);
       closeConnection();
@@ -911,28 +911,10 @@ void
 IMAP4Protocol::mkdir (const KURL & _url, int)
 {
   kdDebug(7116) << "IMAP4::mkdir - " << _url.prettyURL() << endl;
-  QString path = _url.path();
-  int slash = path.findRev('/', (path.at(path.length() - 1) == '/') ?
-    (int)path.length() - 2 : -1);
-  KURL parentUrl = _url;
-  QString newBox;
-  if (slash != -1)
-  {
-    parentUrl.setPath(path.left(slash+1) + ";TYPE=LIST");
-    newBox = path.mid(slash + 1);
-  }
   QString aBox, aSequence, aLType, aSection, aValidity, aDelimiter, aInfo;
-  parseURL(parentUrl, aBox, aSection, aLType, aSequence, aValidity, aDelimiter, aInfo);
-  if ( newBox.isEmpty() ) {
-    newBox = aBox;
-  } else if ( !aBox.isEmpty() ) {
-    if ( !aBox.endsWith( aDelimiter ) )
-      newBox = aBox + aDelimiter + newBox;
-    else
-      newBox = aBox + newBox;
-  }
-  kdDebug(7116) << "IMAP4::mkdir - create " << newBox << endl;
-  imapCommand *cmd = doCommand (imapCommand::clientCreate(newBox));
+  parseURL(_url, aBox, aSection, aLType, aSequence, aValidity, aDelimiter, aInfo);
+  kdDebug(7116) << "IMAP4::mkdir - create " << aBox << endl;
+  imapCommand *cmd = doCommand (imapCommand::clientCreate(aBox));
 
   if (cmd->result () != "OK")
   {
@@ -943,17 +925,22 @@ IMAP4Protocol::mkdir (const KURL & _url, int)
   }
   completeQueue.removeRef (cmd);
 
+  // start a new listing to find the type of the folder
   enum IMAP_TYPE type =
     parseURL(_url, aBox, aSection, aLType, aSequence, aValidity, aDelimiter, aInfo);
   if (type == ITYPE_BOX)
   {
-    if (messageBox(QuestionYesNo,
-      i18n("What do you want to store in this folder?"), i18n("Create Folder"),
-      i18n("&Messages"), i18n("&Subfolders")) == KMessageBox::No)
+    bool ask = ( aInfo.find( "ASKUSER" ) != -1 );
+    if ( ask &&
+        messageBox(QuestionYesNo,
+          i18n("The following folder will be created on the server: %1 "
+               "What do you want to store in this folder?").arg( aBox ),
+          i18n("Create Folder"),
+          i18n("&Messages"), i18n("&Subfolders")) == KMessageBox::No )
     {
-      cmd = doCommand(imapCommand::clientDelete(newBox));
+      cmd = doCommand(imapCommand::clientDelete(aBox));
       completeQueue.removeRef (cmd);
-      cmd = doCommand(imapCommand::clientCreate(newBox + aDelimiter));
+      cmd = doCommand(imapCommand::clientCreate(aBox + aDelimiter));
       if (cmd->result () != "OK")
       {
         error (ERR_COULD_NOT_MKDIR, _url.prettyURL());
@@ -964,7 +951,7 @@ IMAP4Protocol::mkdir (const KURL & _url, int)
     }
   }
 
-  cmd = doCommand(imapCommand::clientSubscribe(newBox));
+  cmd = doCommand(imapCommand::clientSubscribe(aBox));
   completeQueue.removeRef(cmd);
 
   finished ();
@@ -1210,12 +1197,15 @@ IMAP4Protocol::del (const KURL & _url, bool isFile)
  * Capabilities: data = 'c'. Result shipped in infoMessage() signal
  * No-op: data = 'N'
  * Namespace: data = 'n'. Result shipped in infoMessage() signal
+ *                        The format is: section=namespace=delimiter
+ *                        Note that the namespace can be empty
  * Unsubscribe: data = 'U' + URL (KURL)
  * Subscribe: data = 'u' + URL (KURL)
  * Change the status: data = 'S' + URL (KURL) + Flags (QCString)
  * ACL commands: data = 'A' + command + URL (KURL) + command-dependent args
  * AnnotateMore commands: data = 'M' + 'G'et/'S'et + URL + entry + command-dependent args
  * Search: data = 'E' + URL (KURL)
+ * Quota commands: data = 'Q' + 'R'oot/'G'et/'S'et + URL + entry + command-dependent args
  */
 void
 IMAP4Protocol::special (const QByteArray & aData)
@@ -1251,6 +1241,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     imapCommand *cmd = doCommand(imapCommand::clientNoop());
     if (cmd->result () != "OK")
     {
+      kdDebug(7116) << "NOOP did not succeed - connection broken" << endl;
       completeQueue.removeRef (cmd);
       error (ERR_CONNECTION_BROKEN, myHost);
       return;
@@ -1260,9 +1251,9 @@ IMAP4Protocol::special (const QByteArray & aData)
     break;
   }
   case 'n':
-  { 
-    // namespace
-    // separate the entries by , this should be save
+  {
+    // namespace in the form "section=namespace=delimiter"
+    // entries are separated by ,
     infoMessage( imapNamespaces.join(",") );
     finished();
     break;
@@ -1309,7 +1300,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     finished();
     break;
   }
-  case 'A': 
+  case 'A':
   {
     // acl
     int cmd;
@@ -1321,7 +1312,7 @@ IMAP4Protocol::special (const QByteArray & aData)
     }
     break;
   }
-  case 'M': 
+  case 'M':
   {
     // annotatemore
     int cmd;
@@ -1333,7 +1324,19 @@ IMAP4Protocol::special (const QByteArray & aData)
     }
     break;
   }
-  case 'S': 
+  case 'Q':
+  {
+    // quota
+    int cmd;
+    stream >> cmd;
+    if ( hasCapability( "QUOTA" ) ) {
+      specialQuotaCommand( cmd, stream );
+    } else {
+      error( ERR_UNSUPPORTED_ACTION, "QUOTA" );
+    }
+    break;
+  }
+  case 'S':
   {
     // status
     KURL _url;
@@ -1370,7 +1373,38 @@ IMAP4Protocol::special (const QByteArray & aData)
     finished();
     break;
   }
-  case 'E': 
+  case 's':
+  {
+    // seen
+    KURL _url;
+    bool seen;
+    QCString newFlags;
+    stream >> _url >> seen;
+
+    QString aBox, aSequence, aLType, aSection, aValidity, aDelimiter, aInfo;
+    parseURL (_url, aBox, aSection, aLType, aSequence, aValidity, aDelimiter, aInfo);
+    if ( !assureBox(aBox, true) ) // read-only because changing SEEN should be possible even then
+      return;
+
+    imapCommand *cmd;
+    if ( seen )
+      cmd = doCommand( imapCommand::clientStore( aSequence, "+FLAGS.SILENT", "\\SEEN" ) );
+    else
+      cmd = doCommand( imapCommand::clientStore( aSequence, "-FLAGS.SILENT", "\\SEEN" ) );
+
+    if (cmd->result () != "OK")
+    {
+      completeQueue.removeRef (cmd);
+      error(ERR_COULD_NOT_WRITE, i18n("Changing the flags of message %1 "
+                                      "failed.").arg(_url.prettyURL()));
+      return;
+    }
+    completeQueue.removeRef (cmd);
+    finished();
+    break;
+  }
+
+  case 'E':
   {
     // search
     specialSearchCommand( stream );
@@ -1581,6 +1615,50 @@ IMAP4Protocol::specialAnnotateMoreCommand( int command, QDataStream& stream )
   }
 }
 
+void
+IMAP4Protocol::specialQuotaCommand( int command, QDataStream& stream )
+{
+  // All commands start with the URL to the box
+  KURL _url;
+  stream >> _url;
+  QString aBox, aSequence, aLType, aSection, aValidity, aDelimiter, aInfo;
+  parseURL (_url, aBox, aSection, aLType, aSequence, aValidity, aDelimiter, aInfo);
+
+  switch( command ) {
+    case 'R': // GETQUOTAROOT
+      {
+        kdDebug(7116) << "QUOTAROOT " << aBox << endl;
+        imapCommand *cmd = doCommand(imapCommand::clientGetQuotaroot( aBox ) );
+        if (cmd->result () != "OK")
+        {
+          error(ERR_SLAVE_DEFINED, i18n("Retrieving the quota root information on folder %1 "
+                "failed. The server returned: %2")
+              .arg(_url.prettyURL())
+              .arg(cmd->resultInfo()));
+          return;
+        }
+        infoMessage(getResults().join( "\r" ));
+        finished();
+        break;
+      }
+    case 'G': // GETQUOTA
+      {
+        kdDebug(7116) << "GETQUOTA command" << endl;
+        kdWarning(7116) << "UNIMPLEMENTED" << endl;
+        break;
+      }
+    case 'S': // SETQUOTA
+      {
+        kdDebug(7116) << "SETQUOTA command" << endl;
+        kdWarning(7116) << "UNIMPLEMENTED" << endl;
+        break;
+      }
+    default:
+      kdWarning(7116) << "Unknown special quota command:" << command << endl;
+      error( ERR_UNSUPPORTED_ACTION, QString(QChar(command)) );
+  }
+}
+
 void
 IMAP4Protocol::rename (const KURL & src, const KURL & dest, bool overwrite)
 {
@@ -1661,7 +1739,7 @@ IMAP4Protocol::stat (const KURL & _url)
   QString aBox, aSequence, aLType, aSection, aValidity, aDelimiter, aInfo;
   // parseURL with caching
   enum IMAP_TYPE aType =
-    parseURL (_url, aBox, aSection, aLType, aSequence, aValidity, aDelimiter, 
+    parseURL (_url, aBox, aSection, aLType, aSequence, aValidity, aDelimiter,
         aInfo, true);
 
   UDSEntry entry;
@@ -1680,7 +1758,7 @@ IMAP4Protocol::stat (const KURL & _url)
       completeQueue.removeRef(cmd);
       if (!ok)
       {
-        error(ERR_COULD_NOT_STAT, i18n("Unable to close mailbox."));
+        error(ERR_COULD_NOT_STAT, aBox);
         return;
       }
       setState(ISTATE_LOGIN);
@@ -1710,7 +1788,7 @@ IMAP4Protocol::stat (const KURL & _url)
       }
       completeQueue.removeRef (cmd);
       if (found)
-        error(ERR_COULD_NOT_STAT, i18n("Unable to get information about folder %1. The server replied: %2").arg(aBox).arg(cmdInfo));
+        error(ERR_COULD_NOT_STAT, aBox);
       else
         error(KIO::ERR_DOES_NOT_EXIST, aBox);
       return;
@@ -1920,16 +1998,23 @@ bool IMAP4Protocol::makeLogin ()
       } else completeQueue.removeRef(cmd);
     }
 
-    if (!myAuth.isEmpty () && myAuth != "*"
-        && !hasCapability (QString ("AUTH=") + myAuth))
-    {
-      error (ERR_COULD_NOT_LOGIN, i18n("The authentication method %1 is not "
-        "supported by the server.").arg(myAuth));
-      closeConnection();
-      return false;
+    if (myAuth.isEmpty () || myAuth == "*") {
+      if (hasCapability (QString ("LOGINDISABLED"))) {
+        error (ERR_COULD_NOT_LOGIN, i18n("LOGIN is disabled by the server."));
+        closeConnection();
+        return false;
+      }
+    }
+    else {
+      if (!hasCapability (QString ("AUTH=") + myAuth)) {
+        error (ERR_COULD_NOT_LOGIN, i18n("The authentication method %1 is not "
+          "supported by the server.").arg(myAuth));
+        closeConnection();
+        return false;
+      }
     }
 
-    if (  greeting.contains(  QRegExp(  "Cyrus IMAP4 v2.1" ) ) ) {
+    if ( greeting.contains(  QRegExp(  "Cyrus IMAP4 v2.1" ) ) ) {
       removeCapability( "ANNOTATEMORE" );
     }
 
@@ -1952,15 +2037,15 @@ bool IMAP4Protocol::makeLogin ()
         }
       }
       if (!clientLogin (myUser, myPass, resultInfo))
-        error(KIO::ERR_COULD_NOT_LOGIN, i18n("Unable to login. Probably the "
-        "password is wrong.\nThe server replied:\n%1").arg(resultInfo));
+        error(KIO::ERR_COULD_NOT_AUTHENTICATE, i18n("Unable to login. Probably the "
+        "password is wrong.\nThe server %1 replied:\n%2").arg(myHost).arg(resultInfo));
     }
     else
     {
-#ifdef HAVE_LIBSASL2      
+#ifdef HAVE_LIBSASL2
       if (!clientAuthenticate (this, authInfo, myHost, myAuth, mySSL, resultInfo))
-        error(KIO::ERR_COULD_NOT_LOGIN, i18n("Unable to authenticate via %1.\n"
-        "The server replied:\n%2").arg(myAuth).arg(resultInfo));
+        error(KIO::ERR_COULD_NOT_AUTHENTICATE, i18n("Unable to authenticate via %1.\n"
+	"The server %2 replied:\n%3").arg(myAuth).arg(myHost).arg(resultInfo));
       else {
         myUser = authInfo.username;
         myPass = authInfo.password;
@@ -1984,11 +2069,29 @@ bool IMAP4Protocol::makeLogin ()
     if (cmd->result () == "OK")
     {
       QValueListIterator < imapList > it = listResponses.begin();
+      if ( it == listResponses.end() )
+      {
+          // empty answer - this is a buggy imap server
+          // as a fallback we fire a normal listing and take the first answer
+          completeQueue.removeRef (cmd);
+          cmd = doCommand( imapCommand::clientList("", "%") );
+          if (cmd->result () == "OK")
+          {
+              it = listResponses.begin();
+          }
+      }
       if ( it != listResponses.end() )
       {
-        imapNamespaceDelimiter[QString::null] = (*it).hierarchyDelimiter();
-        kdDebug(7116) << "makeLogin - delimiter for empty ns='" << 
+        namespaceToDelimiter[QString::null] = (*it).hierarchyDelimiter();
+        kdDebug(7116) << "makeLogin - delimiter for empty ns='" <<
           (*it).hierarchyDelimiter() << "'" << endl;
+        if ( !hasCapability("NAMESPACE") )
+        {
+          // server does not support namespaces
+          QString nsentry = QString::number( 0 ) + "=="
+            + (*it).hierarchyDelimiter();
+          imapNamespaces.append( nsentry );
+        }
       }
     }
     completeQueue.removeRef (cmd);
@@ -2127,7 +2230,7 @@ IMAP4Protocol::doListEntry (const QString & encodedUrl, int stretch, imapCache *
 
 void
 IMAP4Protocol::doListEntry (const KURL & _url, const QString & myBox,
-                            const imapList & item)
+                            const imapList & item, bool appendPath)
 {
   KURL aURL = _url;
   aURL.setQuery (QString::null);
@@ -2136,11 +2239,11 @@ IMAP4Protocol::doListEntry (const KURL & _url, const QString & myBox,
   int hdLen = item.hierarchyDelimiter().length();
 
   {
-    // mailboxName will be appended to the path!
+    // mailboxName will be appended to the path if appendPath is true
     QString mailboxName = item.name ();
 
     // some beautification
-    if (mailboxName.find (myBox) == 0)
+    if (mailboxName.find (myBox) == 0 && mailboxName.length() > myBox.length())
     {
       mailboxName =
         mailboxName.right (mailboxName.length () - myBox.length ());
@@ -2212,12 +2315,20 @@ IMAP4Protocol::doListEntry (const KURL & _url, const QString & myBox,
       atom.m_uds = UDS_URL;
       QString path = aURL.path();
       atom.m_str = aURL.url (0, 106); // utf-8
-      if (path[path.length() - 1] == '/' && !path.isEmpty() && path != "/")
-        path.truncate(path.length() - 1);
-      if (!path.isEmpty() && path != "/"
-        && path.right(hdLen) != item.hierarchyDelimiter())
-        path += item.hierarchyDelimiter();
-      path += mailboxName;
+      if (appendPath)
+      {
+        if (path[path.length() - 1] == '/' && !path.isEmpty() && path != "/")
+          path.truncate(path.length() - 1);
+        if (!path.isEmpty() && path != "/"
+            && path.right(hdLen) != item.hierarchyDelimiter()) {
+          path += item.hierarchyDelimiter();
+        }
+        path += mailboxName;
+        if (path.upper() == "/INBOX/") {
+            // make sure the client can rely on INBOX
+            path = path.upper();
+        }
+      }
       aURL.setPath(path);
       atom.m_str = aURL.url(0, 106); // utf-8
       atom.m_long = 0;
@@ -2256,9 +2367,9 @@ IMAP4Protocol::parseURL (const KURL & _url, QString & _box,
   // get the delimiter
   QString myNamespace = namespaceForBox( _box );
   kdDebug(7116) << "IMAP4::parseURL - namespace=" << myNamespace << endl;
-  if ( imapNamespaceDelimiter.contains(myNamespace) )
+  if ( namespaceToDelimiter.contains(myNamespace) )
   {
-    _hierarchyDelimiter = imapNamespaceDelimiter[myNamespace];
+    _hierarchyDelimiter = namespaceToDelimiter[myNamespace];
     kdDebug(7116) << "IMAP4::parseURL - delimiter=" << _hierarchyDelimiter << endl;
   }
 
@@ -2306,8 +2417,10 @@ IMAP4Protocol::parseURL (const KURL & _url, QString & _box,
               }
             }
             // if we got no list response for the box see if it's a prefix
-            if ( retVal == ITYPE_UNKNOWN && imapNamespaceDelimiter.contains(_box) )
+            if ( retVal == ITYPE_UNKNOWN &&
+                 namespaceToDelimiter.contains(_box) ) {
               retVal = ITYPE_DIR;
+            }
           } else {
             kdDebug(7116) << "IMAP4::parseURL - got error for " << _box << endl;
           }
@@ -2322,7 +2435,7 @@ IMAP4Protocol::parseURL (const KURL & _url, QString & _box,
     else
       kdDebug(7116) << "IMAP4::parseURL: no login!" << endl;
 
-  } 
+  }
   else // empty box
   {
     // the root is just a dir
@@ -2351,7 +2464,7 @@ IMAP4Protocol::parseURL (const KURL & _url, QString & _box,
   if ( _hierarchyDelimiter.isEmpty() &&
        (_type == "LIST" || _type == "LSUB" || _type == "LSUBNOCHECK") )
   {
-    // this shouldn't happen but when the delimiter is really empty 
+    // this shouldn't happen but when the delimiter is really empty
     // we try to reconstruct it from the URL
     if (!_box.isEmpty())
     {
@@ -2378,11 +2491,12 @@ IMAP4Protocol::outputLine (const QCString & _str, int len)
 
   if (cacheOutput)
   {
-    outputBuffer.open(IO_WriteOnly);
+    if ( !outputBuffer.isOpen() ) {
+      outputBuffer.open(IO_WriteOnly);
+    }
     outputBuffer.at(outputBufferIndex);
     outputBuffer.writeBlock(_str.data(), len);
     outputBufferIndex += len;
-    outputBuffer.close();
     return 0;
   }
 
@@ -2401,8 +2515,9 @@ IMAP4Protocol::outputLine (const QCString & _str, int len)
 void IMAP4Protocol::flushOutput(QString contentEncoding)
 {
   // send out cached data to the application
-  if (outputBufferIndex == 0L)
+  if (outputBufferIndex == 0)
     return;
+  outputBuffer.close();
   outputCache.resize(outputBufferIndex);
   if (decodeContent)
   {
@@ -2425,7 +2540,7 @@ void IMAP4Protocol::flushOutput(QString contentEncoding)
   }
   mProcessedSize += outputBufferIndex;
   processedSize( mProcessedSize );
-  outputBufferIndex = 0L;
+  outputBufferIndex = 0;
   outputCache[0] = '\0';
   outputBuffer.setBuffer(outputCache);
 }
@@ -2441,7 +2556,7 @@ ssize_t IMAP4Protocol::myRead(void *data, ssize_t len)
     return copyLen;
   }
   if (!isConnectionValid()) return 0;
-  waitForResponse( IMAP_TIMEOUT_IN_SECONDS );
+  waitForResponse( responseTimeout() );
   return read(data, len);
 }
 
