@@ -14,7 +14,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/** 
+/**
  * Portions adapted from the SMTP ioslave.
  * Copyright (c) 2000, 2001 Alex Zepeda <jazepeda@pacbell.net>
  * Copyright (c) 2001 Michael Hckel <Michael@Haeckel.Net>
@@ -28,13 +28,17 @@
 # include <config.h>
 #endif
 
+extern "C" {
+#include <sasl/sasl.h>
+}
 #include "sieve.h"
 
 #include <kdebug.h>
 #include <kinstance.h>
 #include <klocale.h>
 #include <kurl.h>
-#include <kio/kdesasl.h>
+#include <kmdcodec.h>
+#include <kglobal.h>
 
 #include <qcstring.h>
 
@@ -42,6 +46,8 @@
 using std::exit;
 #include <sys/stat.h>
 
+#include <kdepimmacros.h>
+
 static const int debugArea = 7122;
 
 static inline
@@ -54,16 +60,27 @@ static inline
 #endif
 
 #define SIEVE_DEFAULT_PORT 2000
-  
+
+static sasl_callback_t callbacks[] = {
+    { SASL_CB_ECHOPROMPT, NULL, NULL },
+    { SASL_CB_NOECHOPROMPT, NULL, NULL },
+    { SASL_CB_GETREALM, NULL, NULL },
+    { SASL_CB_USER, NULL, NULL },
+    { SASL_CB_AUTHNAME, NULL, NULL },
+    { SASL_CB_PASS, NULL, NULL },
+    { SASL_CB_CANON_USER, NULL, NULL },
+    { SASL_CB_LIST_END, NULL, NULL }
+};
+
 static const unsigned int SIEVE_DEFAULT_RECIEVE_BUFFER = 512;
 
 using namespace KIO;
 extern "C"
 {
-	int kdemain(int argc, char **argv)
+	KDE_EXPORT int kdemain(int argc, char **argv)
 	{
 		KInstance instance("kio_sieve" );
-		
+
 		ksDebug() << "*** Starting kio_sieve " << endl;
 
 		if (argc != 4) {
@@ -71,9 +88,16 @@ extern "C"
 			exit(-1);
 		}
 
+    if ( sasl_client_init( NULL ) != SASL_OK ) {
+      fprintf(stderr, "SASL library initialization failed!\n");
+      ::exit (-1);
+    }
+
 		kio_sieveProtocol slave(argv[2], argv[3]);
 		slave.dispatchLoop();
 
+    sasl_done();
+
 		ksDebug() << "*** kio_sieve Done" << endl;
 		return 0;
 	}
@@ -205,17 +229,17 @@ void kio_sieveProtocol::openConnection()
 bool kio_sieveProtocol::parseCapabilities(bool requestCapabilities/* = false*/)
 {
 	ksDebug() << k_funcinfo << endl;
-	
+
 	// Setup...
 	bool ret = false;
-	
+
 	if (requestCapabilities) {
 		sendData("CAPABILITY");
 	}
 
 	while (receiveData()) {
 		ksDebug() << "Looping receive" << endl;
-		
+
 		if (r.getType() == kio_sieveResponse::ACTION) {
 			if ( r.getAction().contains("ok", false) != -1 ) {
 				ksDebug() << "Sieve server ready & awaiting authentication." << endl;
@@ -246,12 +270,12 @@ bool kio_sieveProtocol::parseCapabilities(bool requestCapabilities/* = false*/)
 			ksDebug() << "Server supports TLS" << endl;
 			m_supportsTLS = true;
 			setMetaData("tlsSupported", "true");
-			
+
 		} else {
-			ksDebug() << "Unrecognised key." << endl;
+			ksDebug() << "Unrecognised key " << r.getKey() << endl;
 		}
 	}
-	
+
 	if (!m_supportsTLS) {
 		setMetaData("tlsSupported", "false");
 	}
@@ -259,6 +283,39 @@ bool kio_sieveProtocol::parseCapabilities(bool requestCapabilities/* = false*/)
 	return ret;
 }
 
+
+/* ---------------------------------------------------------------------------------- */
+/**
+ * Checks if connection parameters (currently - auth method) have changed.
+ * If it it, close the current connection
+ */
+void kio_sieveProtocol::changeCheck( const KURL &url )
+{
+	QString auth;
+
+	if (!metaData("sasl").isEmpty())
+		auth = metaData("sasl").upper();
+	else {
+		QString query = url.query();
+		if ( query.startsWith("?") ) query.remove( 0, 1 );
+		QStringList q = QStringList::split( ",", query );
+		QStringList::iterator it;
+
+		for ( it = q.begin(); it != q.end(); ++it ) {
+			if ( ( (*it).section('=',0,0) ).lower() == "x-mech" ) {
+				auth = ( (*it).section('=',1) ).upper();
+				break;
+			}
+		}
+	}
+	ksDebug() << "auth: " << auth << " m_sAuth: " << m_sAuth << endl;
+	if ( m_sAuth != auth ) {
+		m_sAuth = auth;
+		if ( isConnectionValid() )
+			disconnect();
+	}
+}
+
 /* ---------------------------------------------------------------------------------- */
 /**
  * Connects to the server.
@@ -267,7 +324,7 @@ bool kio_sieveProtocol::parseCapabilities(bool requestCapabilities/* = false*/)
 bool kio_sieveProtocol::connect(bool useTLSIfAvailable)
 {
 	ksDebug() << k_funcinfo << endl;
-	
+
 	if (isConnectionValid()) return true;
 
 	infoMessage(i18n("Connecting to %1...").arg( m_sServer));
@@ -276,7 +333,7 @@ bool kio_sieveProtocol::connect(bool useTLSIfAvailable)
 		error(ERR_CONNECTION_BROKEN, i18n("The connection to the server was lost."));
 		return false;
 	}
-	
+
 	setBlockConnection(true);
 
 	if (!connectToHost(m_sServer, m_iPort, true)) {
@@ -300,7 +357,7 @@ bool kio_sieveProtocol::connect(bool useTLSIfAvailable)
 			if (retval == 1) {
 				ksDebug() << "TLS enabled successfully." << endl;
 				// reparse capabilities:
-				parseCapabilities(true);
+				parseCapabilities(false);
 			} else {
 				ksDebug() << "TLS initiation failed, code " << retval << endl;
 				disconnect(true);
@@ -348,7 +405,7 @@ void kio_sieveProtocol::disconnect(bool forcibly)
 /*void kio_sieveProtocol::slave_status()
 {
 	slaveStatus(isConnectionValid() ? m_sServer : "", isConnectionValid());
-	
+
 	finished();
 }*/
 
@@ -384,6 +441,7 @@ void kio_sieveProtocol::special(const QByteArray &data)
 /* ---------------------------------------------------------------------------------- */
 bool kio_sieveProtocol::activate(const KURL& url)
 {
+	changeCheck( url );
 	if (!connect())
 		return false;
 
@@ -445,6 +503,7 @@ static void append_lf2crlf( QByteArray & out, const QByteArray & in ) {
 
 void kio_sieveProtocol::put(const KURL& url, int /*permissions*/, bool /*overwrite*/, bool /*resume*/)
 {
+	changeCheck( url );
 	if (!connect())
 		return;
 
@@ -473,7 +532,7 @@ void kio_sieveProtocol::put(const KURL& url, int /*permissions*/, bool /*overwri
 	}
 
 	// script size
-	int bufLen = (int)data.size() - 1;
+	int bufLen = (int)data.size();
 	totalSize(bufLen);
 
 	// timsieved 1.1.0:
@@ -540,7 +599,7 @@ void kio_sieveProtocol::put(const KURL& url, int /*permissions*/, bool /*overwri
 
 	if (operationSuccessful())
 		ksDebug() << "Script upload complete." << endl;
-	
+
 	else {
 		/* The managesieve server parses received scripts and rejects
 		 * scripts which are not syntactically correct. Here we expect
@@ -553,7 +612,7 @@ void kio_sieveProtocol::put(const KURL& url, int /*permissions*/, bool /*overwri
 			// send the extra message off for re-processing
 			receiveData(false, &extra);
 
-			if (r.getAction() == kio_sieveResponse::QUANTITY) {
+			if (r.getType() == kio_sieveResponse::QUANTITY) {
 				// length of the error message
 				uint len = r.getQuantity();
 
@@ -568,7 +627,12 @@ void kio_sieveProtocol::put(const KURL& url, int /*permissions*/, bool /*overwri
 
 				// clear the rest of the incoming data
 				receiveData();
-			} else
+			} else if (r.getType() == kio_sieveResponse::KEY_VAL_PAIR) {
+				error(ERR_INTERNAL_SERVER,
+						i18n("The script did not upload successfully.\n"
+							"This is probably due to errors in the script.\n"
+							"The server responded:\n%1").arg(r.getKey()));
+			} else 
 				error(ERR_INTERNAL_SERVER,
 					i18n("The script did not upload successfully.\n"
 						"The script may contain errors."));
@@ -605,6 +669,7 @@ static void inplace_crlf2lf( QByteArray & in ) {
 /* ---------------------------------------------------------------------------------- */
 void kio_sieveProtocol::get(const KURL& url)
 {
+	changeCheck( url );
 	if (!connect())
 		return;
 
@@ -624,25 +689,41 @@ void kio_sieveProtocol::get(const KURL& url)
 
 	if (receiveData() && r.getType() == kio_sieveResponse::QUANTITY) {
 		// determine script size
-		ssize_t bufLen = r.getQuantity();
-		totalSize( bufLen );
+		ssize_t total_len = r.getQuantity();
+		totalSize( total_len );
 
-		QByteArray dat( bufLen );
-		ssize_t readLen = read( dat.data(), dat.size() );
-		
-		if (readLen != bufLen) {
-			error(ERR_COULD_NOT_READ, i18n("Network error."));
-			disconnect(true);
-			return;
-		}
+		int recv_len = 0;
+		do {
+		  // wait for data...
+		  if ( !waitForResponse( 600 ) ) {
+		    error( KIO::ERR_SERVER_TIMEOUT, m_sServer );
+		    disconnect( true );
+		    return;
+		  }
 
-		inplace_crlf2lf( dat );
-		// send data to slaveinterface
-		data(dat);
-		processedSize(readLen);
-		data(QByteArray());
+		  // ...read data...
+		  // Only read as much as we need, otherwise we slurp in the OK that
+		  // operationSuccessful() is expecting below.
+		  QByteArray dat( kMin( total_len - recv_len, ssize_t(64 * 1024 )) );
+		  ssize_t this_recv_len = read( dat.data(), dat.size() );
+
+		  if ( this_recv_len < 1 && !isConnectionValid() ) {
+		    error( KIO::ERR_CONNECTION_BROKEN, m_sServer );
+		    disconnect( true );
+		    return;
+		  }
+
+		  dat.resize( this_recv_len );
+		  inplace_crlf2lf( dat );
+		  // send data to slaveinterface
+		  data( dat );
+
+		  recv_len += this_recv_len;
+		  processedSize( recv_len );
+		} while ( recv_len < total_len );
 
 		infoMessage(i18n("Finishing up...") );
+		data(QByteArray());
 
 		if (operationSuccessful())
 			ksDebug() << "Script retrieval complete." << endl;
@@ -665,6 +746,7 @@ void kio_sieveProtocol::del(const KURL &url, bool isfile)
 		return;
 	}
 
+	changeCheck( url );
 	if (!connect())
 		return;
 
@@ -688,7 +770,7 @@ void kio_sieveProtocol::del(const KURL &url, bool isfile)
 	}
 
 	infoMessage(i18n("Done."));
-	
+
 	finished();
 }
 
@@ -705,12 +787,17 @@ void kio_sieveProtocol::chmod(const KURL& url, int permissions)
     error(ERR_CANNOT_CHMOD, i18n("Cannot chmod to anything but 0700 (active) or 0600 (inactive script)."));
     return;
   }
-  
+
   finished();
 }
 
+#if defined(_AIX) && defined(stat)
+#undef stat
+#endif
+
 void kio_sieveProtocol::stat(const KURL& url)
 {
+	changeCheck( url );
 	if (!connect())
 		return;
 
@@ -740,7 +827,7 @@ void kio_sieveProtocol::stat(const KURL& url)
 
 		while(receiveData()) {
 			if (r.getType() == kio_sieveResponse::ACTION) {
-				if (r.getAction().contains("OK", false) == 1) 
+				if (r.getAction().contains("OK", false) == 1)
 					// Script list completed
 					break;
 
@@ -780,8 +867,9 @@ void kio_sieveProtocol::stat(const KURL& url)
 	finished();
 }
 
-void kio_sieveProtocol::listDir(const KURL& /*url*/)
+void kio_sieveProtocol::listDir(const KURL& url)
 {
+	changeCheck( url );
 	if (!connect())
 		return;
 
@@ -832,8 +920,66 @@ void kio_sieveProtocol::listDir(const KURL& /*url*/)
 }
 
 /* ---------------------------------------------------------------------------------- */
+bool kio_sieveProtocol::saslInteract( void *in, AuthInfo &ai )
+{
+  ksDebug() << "sasl_interact" << endl;
+  sasl_interact_t *interact = ( sasl_interact_t * ) in;
+
+  //some mechanisms do not require username && pass, so it doesn't need a popup
+  //window for getting this info
+  for ( ; interact->id != SASL_CB_LIST_END; interact++ ) {
+    if ( interact->id == SASL_CB_AUTHNAME ||
+         interact->id == SASL_CB_PASS ) {
+
+	    if (m_sUser.isEmpty() || m_sPass.isEmpty()) {
+		    if (!openPassDlg(ai)) {
+			    error(ERR_ABORTED, i18n("No authentication details supplied."));
+			    return false;
+		    }
+        m_sUser = ai.username;
+        m_sPass = ai.password;
+	    }
+      break;
+    }
+  }
+
+  interact = ( sasl_interact_t * ) in;
+  while( interact->id != SASL_CB_LIST_END ) {
+    ksDebug() << "SASL_INTERACT id: " << interact->id << endl;
+    switch( interact->id ) {
+      case SASL_CB_USER:
+      case SASL_CB_AUTHNAME:
+        ksDebug() << "SASL_CB_[AUTHNAME|USER]: '" << m_sUser << "'" << endl;
+        interact->result = strdup( m_sUser.utf8() );
+        interact->len = strlen( (const char *) interact->result );
+        break;
+      case SASL_CB_PASS:
+        ksDebug() << "SASL_CB_PASS: [hidden] " << endl;
+        interact->result = strdup( m_sPass.utf8() );
+        interact->len = strlen( (const char *) interact->result );
+        break;
+      default:
+        interact->result = NULL; interact->len = 0;
+        break;
+    }
+    interact++;
+  }
+  return true;
+}
+
+#define SASLERROR  error(ERR_COULD_NOT_AUTHENTICATE, i18n("An error occurred during authentication: %1").arg( \
+      QString::fromUtf8( sasl_errdetail( conn ) )));
+
 bool kio_sieveProtocol::authenticate()
 {
+  int result;
+  sasl_conn_t *conn = NULL;
+  sasl_interact_t *client_interact = NULL;
+  const char *out = NULL;
+  uint outlen;
+  const char *mechusing = NULL;
+    QByteArray challenge, tmp;
+
 	/* Retrieve authentication details from user.
 	 * Note: should this require realm as well as user & pass details
 	 * before it automatically skips the prompt?
@@ -845,86 +991,132 @@ bool kio_sieveProtocol::authenticate()
 	ai.username = m_sUser;
 	ai.password = m_sPass;
 	ai.keepPassword = true;
+  ai.caption = i18n("Sieve Authentication Details");
+  ai.comment = i18n("Please enter your authentication details for your sieve account "
+	  "(usually the same as your email password):");
 
-	bool cachePositive = checkCachedAuthentication(ai);
-	if (!cachePositive && (m_sUser.isEmpty() || m_sPass.isEmpty())) {
-		ai.caption = i18n("Sieve Authentication Details");
-		ai.comment = i18n("Please enter your authentication details for your sieve account "
-								"(usually the same as your email password):");
-		if (!openPassDlg(ai)) {
-			error(ERR_ABORTED, i18n("No authentication details supplied."));
-			return false;
-		}
-	}
+  result = sasl_client_new( "sieve",
+                       m_sServer.latin1(),
+                       0, 0, callbacks, 0, &conn );
 
-	KDESasl sasl(ai.username, ai.password, "sieve");
+  if ( result != SASL_OK ) {
+    ksDebug() << "sasl_client_new failed with: " << result << endl;
+    SASLERROR
+    return false;
+  }
 
-	QStrIList strList;
-	//strList.append("CRAM-MD5");
-	if (!metaData("sasl").isEmpty())
-		strList.append(metaData("sasl").latin1());
-	else
-		for (uint i = 0; i < m_sasl_caps.count(); ++i) strList.append(m_sasl_caps[i].latin1());
+	QStringList strList;
+//	strList.append("NTLM");
 
-	m_auth_method = sasl.chooseMethod(strList);
-	ksDebug() << "Preferred authentication method is " << m_auth_method << "." << endl;
+  if ( !m_sAuth.isEmpty() )
+    strList.append( m_sAuth );
+  else
+    strList = m_sasl_caps;
 
-	if (m_auth_method.isEmpty()) {
-		ksDebug() << "No authentication available." << endl;
-		error(ERR_COULD_NOT_AUTHENTICATE, i18n("No compatible authentication methods found."));
+  do {
+    result = sasl_client_start(conn, strList.join(" ").latin1(), &client_interact,
+                       &out, &outlen, &mechusing);
+
+    if (result == SASL_INTERACT)
+      if ( !saslInteract( client_interact, ai ) ) {
+        sasl_dispose( &conn );
+        return false;
+      };
+  } while ( result == SASL_INTERACT );
+
+  if ( result != SASL_CONTINUE && result != SASL_OK ) {
+    ksDebug() << "sasl_client_start failed with: " << result << endl;
+    SASLERROR
+    sasl_dispose( &conn );
+    return false;
+  }
+
+	ksDebug() << "Preferred authentication method is " << mechusing << "." << endl;
+
+  QString firstCommand = "AUTHENTICATE \"" + QString::fromLatin1( mechusing ) + "\"";
+  tmp.setRawData( out, outlen );
+  KCodecs::base64Encode( tmp, challenge );
+  tmp.resetRawData( out, outlen );
+  if ( !challenge.isEmpty() ) {
+    firstCommand += " \"";
+    firstCommand += QString::fromLatin1( challenge.data(), challenge.size() );
+    firstCommand += "\"";
+  }
+
+  ksDebug() << "firstCommand: " << firstCommand << endl;
+	if (!sendData( firstCommand.latin1() ))
 		return false;
-	}
 
-	if (!sendData("AUTHENTICATE \"" + m_auth_method + "\""))
-		return false;
-	
 	QCString command;
-	
+
 	do {
 		receiveData();
-		
+
 		if (operationResult() != OTHER)
 			break;
-		
+
 		ksDebug() << "Challenge len  " << r.getQuantity() << endl;
 
 		if (r.getType() != kio_sieveResponse::QUANTITY) {
-			error(ERR_UNSUPPORTED_PROTOCOL,
+      sasl_dispose( &conn );
+			error(ERR_SLAVE_DEFINED,
 					i18n("A protocol error occurred during authentication.\n"
-							"Choose a different authentication method to %1.").arg(m_auth_method));
+							"Choose a different authentication method to %1.").arg(mechusing));
 			return false;
 		}
 
 		uint qty = r.getQuantity();
 
 		receiveData();
-		ksDebug() << "Challenge:  [" << r.getAction() << "]." << endl;
 
 		if (r.getType() != kio_sieveResponse::ACTION && r.getAction().length() != qty) {
+      sasl_dispose( &conn );
 			error(ERR_UNSUPPORTED_PROTOCOL,
 					i18n("A protocol error occurred during authentication.\n"
-							"Choose a different authentication method to %1.").arg(m_auth_method));
+							"Choose a different authentication method to %1.").arg(mechusing));
 			return false;
 		}
 
-		command = sasl.getResponse(r.getAction());
+    tmp.setRawData( r.getAction().data(), qty );
+    KCodecs::base64Decode( tmp, challenge );
+    tmp.resetRawData( r.getAction().data(), qty );
+//		ksDebug() << "S:  [" << r.getAction() << "]." << endl;
+//		ksDebug() << "S-1:  [" << QCString(challenge.data(), challenge.size()+1) << "]." << endl;
 
-		if (!command.isEmpty()) {
-// 			if (m_auth_method == "CRAM-MD5") {
-// 				QCString len = QString("{%1+}").arg(command.length()).latin1();
-// 				sendData(len);
-// 				sendData(command);
-// 			} else {
-				sendData("\"" + command + "\"");
-/*			}*/
-		}
+    do {
+      result = sasl_client_step(conn, challenge.isEmpty() ? 0 : challenge.data(),
+                                  challenge.size(),
+                                  &client_interact,
+                                  &out, &outlen);
 
-	} while (!command.isEmpty());
+      if (result == SASL_INTERACT)
+        if ( !saslInteract( client_interact, ai ) ) {
+          sasl_dispose( &conn );
+          return false;
+        };
+    } while ( result == SASL_INTERACT );
+
+    ksDebug() << "sasl_client_step: " << result << endl;
+    if ( result != SASL_CONTINUE && result != SASL_OK ) {
+      ksDebug() << "sasl_client_step failed with: " << result << endl;
+      SASLERROR
+      sasl_dispose( &conn );
+      return false;
+    }
+
+    tmp.setRawData( out, outlen );
+    KCodecs::base64Encode( tmp, challenge );
+    tmp.resetRawData( out, outlen );
+    sendData("\"" + QCString( challenge.data(), challenge.size()+1 ) + "\"");
+//    ksDebug() << "C:  [" << QCString(challenge.data(), challenge.size()+1) << "]." << endl;
+//    ksDebug() << "C-1:  [" << out << "]." << endl;
+  } while ( true );
 
 	ksDebug() << "Challenges finished." << endl;
+  sasl_dispose( &conn );
+
 	if (operationResult() == OK) {
 		// Authentication succeeded.
-		if (!cachePositive) cacheAuthentication(ai);
 		return true;
 	} else {
 		// Authentication failed.
@@ -997,7 +1189,7 @@ bool kio_sieveProtocol::receiveData(bool waitForData, QCString *reparse)
 		  {
 			// expecting {quantity}
 			start = 0;
-			end = interpret.find('}', start + 1);
+			end = interpret.find('}', start + 1)-1;
 
 			bool ok = false;
 			r.setQuantity(interpret.mid(start + 1, end - start - 1).toUInt( &ok ));
@@ -1076,6 +1268,6 @@ int kio_sieveProtocol::operationResult()
 			return BYE;
 		}
 	}
-	
+
 	return OTHER;
 }
