@@ -84,24 +84,23 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
   mDestFolder = folder? folder->folder() : 0;
   // refcount++
   if (folder) {
-    folder->open();
+    folder->open("imapjobdest");
   }
   KMFolder *msg_parent = msg->parent();
   if (msg_parent) {
-    if (!folder || folder!= msg_parent->storage()) {
-      msg_parent->open();
-    }
+    msg_parent->open("imapjobsrc");
   }
   mSrcFolder = msg_parent;
   // If there is a destination folder, this is a copy, move or put to an
   // imap folder, use its account for keeping track of the job. Otherwise,
   // this is a get job and the src folder is an imap one. Use its account
   // then.
-  KMAcctImap *account;
+  KMAcctImap *account = 0;
   if (folder) {
     account = folder->account();
   } else {
-    account = static_cast<KMFolderImap*>(msg_parent->storage())->account();
+    if ( msg_parent && msg_parent->storage() )
+      account = static_cast<KMFolderImap*>(msg_parent->storage())->account();
   }
   if ( !account ||
        account->makeConnection() == ImapAccountBase::Error ) {
@@ -127,7 +126,7 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
       url.setPath( folder->imapPath() + ";SECTION=" + flags );
       ImapAccountBase::jobData jd;
       jd.parent = 0; jd.offset = 0; jd.done = 0;
-      jd.total = ( curMsg->msgSizeServer() > 0 ) ? 
+      jd.total = ( curMsg->msgSizeServer() > 0 ) ?
         curMsg->msgSizeServer() : curMsg->msgSize();
       jd.msgList.append( curMsg );
       QCString cstr( curMsg->asString() );
@@ -152,7 +151,7 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
           mParentProgressItem,
           "ImapJobUploading"+ProgressManager::getUniqueID(),
           i18n("Uploading message data"),
-          curMsg->subject(),
+          QStyleSheet::escape( curMsg->subject() ),
           true,
           account->useSSL() || account->useTLS() );
       jd.progressItem->setTotalItems( jd.total );
@@ -192,8 +191,8 @@ void ImapJob::init( JobType jt, QString sets, KMFolderImap* folder,
                           "ImapJobCopyMove"+ProgressManager::getUniqueID(),
                           i18n("Server operation"),
                           i18n("Source folder: %1 - Destination folder: %2")
-                            .arg( msg_parent->prettyURL(), 
-                                  mDestFolder->prettyURL() ),
+                            .arg( QStyleSheet::escape( msg_parent->prettyURL() ),
+                                  QStyleSheet::escape( mDestFolder->prettyURL() ) ),
                           true,
                           account->useSSL() || account->useTLS() );
     jd.progressItem->setTotalItems( jd.total );
@@ -223,29 +222,8 @@ ImapJob::~ImapJob()
   if ( mDestFolder )
   {
     KMAcctImap *account = static_cast<KMFolderImap*>(mDestFolder->storage())->account();
-    if ( account &&  mJob ) {
-      ImapAccountBase::JobIterator it = account->findJob( mJob );
-      if ( it != account->jobsEnd() ) {
-        if( (*it).progressItem ) {
-          (*it).progressItem->setComplete();
-          (*it).progressItem = 0;
-        }
-        if ( !(*it).msgList.isEmpty() ) {
-          for ( QPtrListIterator<KMMessage> mit( (*it).msgList ); mit.current(); ++mit )
-            mit.current()->setTransferInProgress( false );
-        }
-      }
-      account->removeJob( mJob );
-    }
-    account->mJobList.remove( this );
-    mDestFolder->close();
-  }
-
-  if ( mSrcFolder ) {
-    if (!mDestFolder || mDestFolder != mSrcFolder) {
-      if (! (mSrcFolder->folderType() == KMFolderTypeImap) ) return;
-      KMAcctImap *account = static_cast<KMFolderImap*>(mSrcFolder->storage())->account();
-      if ( account && mJob ) {
+    if ( account ) {
+      if ( mJob ) {
         ImapAccountBase::JobIterator it = account->findJob( mJob );
         if ( it != account->jobsEnd() ) {
           if( (*it).progressItem ) {
@@ -257,11 +235,36 @@ ImapJob::~ImapJob()
               mit.current()->setTransferInProgress( false );
           }
         }
-        account->removeJob( mJob ); // remove the associated kio job
+        account->removeJob( mJob );
       }
-      account->mJobList.remove( this ); // remove the folderjob
+      account->mJobList.remove( this );
     }
-    mSrcFolder->close();
+    mDestFolder->close("imapjobdest");
+  }
+
+  if ( mSrcFolder ) {
+    if (!mDestFolder || mDestFolder != mSrcFolder) {
+      if (! (mSrcFolder->folderType() == KMFolderTypeImap) ) return;
+      KMAcctImap *account = static_cast<KMFolderImap*>(mSrcFolder->storage())->account();
+      if ( account ) {
+        if ( mJob ) {
+          ImapAccountBase::JobIterator it = account->findJob( mJob );
+          if ( it != account->jobsEnd() ) {
+            if( (*it).progressItem ) {
+              (*it).progressItem->setComplete();
+              (*it).progressItem = 0;
+            }
+            if ( !(*it).msgList.isEmpty() ) {
+              for ( QPtrListIterator<KMMessage> mit( (*it).msgList ); mit.current(); ++mit )
+                mit.current()->setTransferInProgress( false );
+            }
+          }
+          account->removeJob( mJob ); // remove the associated kio job
+        }
+        account->mJobList.remove( this ); // remove the folderjob
+      }
+    }
+    mSrcFolder->close("imapjobsrc");
   }
 }
 
@@ -270,15 +273,15 @@ ImapJob::~ImapJob()
 void ImapJob::slotGetNextMessage()
 {
   KMMessage *msg = mMsgList.first();
-  KMFolderImap *msgParent = static_cast<KMFolderImap*>(msg->storage());
-  KMAcctImap *account = msgParent->account();
-  if ( msg->UID() == 0 )
+  KMFolderImap *msgParent = msg ? static_cast<KMFolderImap*>(msg->storage()) : 0;
+  if ( !msgParent || !msg || msg->UID() == 0 )
   {
     // broken message
     emit messageRetrieved( 0 );
     deleteLater();
     return;
   }
+  KMAcctImap *account = msgParent->account();
   KURL url = account->getUrl();
   QString path = msgParent->imapPath() + ";UID=" + QString::number(msg->UID());
   ImapAccountBase::jobData jd;
@@ -306,12 +309,12 @@ void ImapJob::slotGetNextMessage()
 //  kdDebug(5006) << "ImapJob::slotGetNextMessage - retrieve " << url.path() << endl;
   // protect the message, otherwise we'll get crashes afterwards
   msg->setTransferInProgress( true );
-  const QString escapedSubject = QStyleSheet::escape( msg->subject() );
   jd.progressItem = ProgressManager::createProgressItem(
                           mParentProgressItem,
                           "ImapJobDownloading"+ProgressManager::getUniqueID(),
                           i18n("Downloading message data"),
-                          i18n("Message with subject: ") + escapedSubject,
+                          i18n("Message with subject: ") +
+                          QStyleSheet::escape( msg->subject() ),
                           true,
                           account->useSSL() || account->useTLS() );
   connect ( jd.progressItem, SIGNAL( progressItemCanceled( KPIM::ProgressItem*)),
@@ -391,7 +394,7 @@ void ImapJob::slotGetMessageResult( KIO::Job * job )
         dataSize = Util::crlf2lf( (*it).data.data(), dataSize ); // always <=
         (*it).data.resize( dataSize );
 
-        // During the construction of the message from the byteArray it does 
+        // During the construction of the message from the byteArray it does
         // not have a uid. Therefore we have to make sure that no connected
         // slots are called, since they would operate on uid == 0.
         msg->parent()->storage()->blockSignals( true );
@@ -445,7 +448,7 @@ void ImapJob::slotGetMessageResult( KIO::Job * job )
       parent->ignoreJobsForMessage( msg );
       int idx = parent->find( msg );
       if (idx != -1) parent->removeMsg( idx, true );
-      // the removeMsg will unGet the message, which will delete all 
+      // the removeMsg will unGet the message, which will delete all
       // jobs, including this one
       return;
     }
@@ -497,6 +500,12 @@ void ImapJob::slotGetBodyStructureResult( KIO::Job * job )
 void ImapJob::slotPutMessageDataReq( KIO::Job *job, QByteArray &data )
 {
   KMAcctImap *account = static_cast<KMFolderImap*>(mDestFolder->storage())->account();
+  if ( !account )
+  {
+    emit finished();
+    deleteLater();
+    return;
+  }
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
 
@@ -517,6 +526,12 @@ void ImapJob::slotPutMessageDataReq( KIO::Job *job, QByteArray &data )
 void ImapJob::slotPutMessageResult( KIO::Job *job )
 {
   KMAcctImap *account = static_cast<KMFolderImap*>(mDestFolder->storage())->account();
+  if ( !account )
+  {
+    emit finished();
+    deleteLater();
+    return;
+  }
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
   bool deleteMe = false;
@@ -557,6 +572,12 @@ void ImapJob::slotCopyMessageInfoData(KIO::Job * job, const QString & data)
 {
   KMFolderImap * imapFolder = static_cast<KMFolderImap*>(mDestFolder->storage());
   KMAcctImap *account = imapFolder->account();
+  if ( !account )
+  {
+    emit finished();
+    deleteLater();
+    return;
+  }
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
 
@@ -590,6 +611,12 @@ void ImapJob::slotPutMessageInfoData(KIO::Job *job, const QString &data)
 {
   KMFolderImap * imapFolder = static_cast<KMFolderImap*>(mDestFolder->storage());
   KMAcctImap *account = imapFolder->account();
+  if ( !account )
+  {
+    emit finished();
+    deleteLater();
+    return;
+  }
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
 
@@ -608,6 +635,12 @@ void ImapJob::slotPutMessageInfoData(KIO::Job *job, const QString &data)
 void ImapJob::slotCopyMessageResult( KIO::Job *job )
 {
   KMAcctImap *account = static_cast<KMFolderImap*>(mDestFolder->storage())->account();
+  if ( !account )
+  {
+    emit finished();
+    deleteLater();
+    return;
+  }
   ImapAccountBase::JobIterator it = account->findJob( job );
   if ( it == account->jobsEnd() ) return;
 
