@@ -1,5 +1,6 @@
 /*
     This file is part of libkcal.
+
     Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>
     Copyright (C) 2004 Reinhold Kainhofer <reinhold@kainhofer.com>
 
@@ -15,8 +16,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <kdebug.h>
@@ -95,6 +96,8 @@ FreeBusy::FreeBusy( Calendar *calendar, const QDateTime &start, const QDateTime
 
       if( event->doesRecur() ) {
         if ( event->isMultiDay() ) {
+// FIXME: This doesn't work for sub-daily recurrences or recurrences with
+//        a different time than the original event.
           extraDays = event->dtStart().date().daysTo(event->dtEnd().date());
           for ( x = 0; x <= extraDays; ++x ) {
             if ( event->recursOn(day.addDays(-x))) {
@@ -142,7 +145,7 @@ QDateTime FreeBusy::dtEnd() const
   return mDtEnd;
 }
 
-QValueList<Period> FreeBusy::busyPeriods() const
+PeriodList FreeBusy::busyPeriods() const
 {
   return mBusyPeriods;
 }
@@ -153,7 +156,7 @@ bool FreeBusy::addLocalPeriod(const QDateTime &eventStart, const QDateTime &even
 
   //Check to see if the start *or* end of the event is
   //between the start and end of the freebusy dates.
-  if ( !( ( ( dtStart().secsTo(eventStart) >= 0 ) && 
+  if ( !( ( ( dtStart().secsTo(eventStart) >= 0 ) &&
             ( eventStart.secsTo(dtEnd()) >= 0 ) )
        || ( ( dtStart().secsTo(eventEnd) >= 0 ) &&
             ( eventEnd.secsTo(dtEnd()) >= 0 ) ) ) )
@@ -177,48 +180,45 @@ bool FreeBusy::addLocalPeriod(const QDateTime &eventStart, const QDateTime &even
   return true;
 }
 
-FreeBusy::FreeBusy(QValueList<Period> busyPeriods)
+FreeBusy::FreeBusy( PeriodList busyPeriods)
 {
   mBusyPeriods = busyPeriods;
 }
 
 void FreeBusy::sortList()
 {
-  typedef QValueList<Period> PeriodList;
+  qHeapSort( mBusyPeriods );
+  return;
+}
 
-  PeriodList::Iterator tmpPeriod, earlyPeriod;
-  PeriodList sortedList;
-  QDateTime earlyTime;
-
-  while( mBusyPeriods.count() > 0 ) {
-    earlyTime=(*mBusyPeriods.begin()).start();
-    for ( tmpPeriod = mBusyPeriods.begin(); tmpPeriod != mBusyPeriods.end(); ++tmpPeriod ) {
-      if (earlyTime.secsTo((*tmpPeriod).start()) <= 0) {
-        earlyTime=(*tmpPeriod).start();
-        earlyPeriod=tmpPeriod;
-      }
-    }
-    //Move tmpPeriod to sortedList
-    Period tmpPeriod = (*earlyPeriod);
-    sortedList.append( tmpPeriod );
-    mBusyPeriods.remove( earlyPeriod );
-  }
-  mBusyPeriods=sortedList;
+void FreeBusy::addPeriods(const PeriodList &list )
+{
+  mBusyPeriods += list;
+  sortList();
 }
 
 void FreeBusy::addPeriod(const QDateTime &start, const QDateTime &end)
 {
-  Period p(start, end);
-  mBusyPeriods.append( p );
-
+  mBusyPeriods.append( Period(start, end) );
   sortList();
 }
 
 void FreeBusy::addPeriod( const QDateTime &start, const Duration &dur )
 {
-  Period p(start, dur);
-  mBusyPeriods.append( p );
-
+  mBusyPeriods.append( Period(start, dur) );
   sortList();
 }
 
+void FreeBusy::merge( FreeBusy *freeBusy )
+{
+  if ( freeBusy->dtStart() < dtStart() )
+    setDtStart( freeBusy->dtStart() );
+
+  if ( freeBusy->dtEnd() > dtEnd() )
+    setDtEnd( freeBusy->dtEnd() );
+
+  QValueList<Period> periods = freeBusy->busyPeriods();
+  QValueList<Period>::ConstIterator it;
+  for ( it = periods.begin(); it != periods.end(); ++it )
+    addPeriod( (*it).start(), (*it).end() );
+}
