@@ -43,7 +43,7 @@ namespace KPIM {
 
         @param vCard The vCard in string representation.
        */
-      virtual void importVCard( const QString& vCard ) = 0;
+      virtual void importVCardFromData( const QString& vCard ) = 0;
 
       /**
         This method will add a vcard to the address book.
