@@ -16,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     In addition, as a special exception, the copyright holders give
     permission to link the code of this program with any edition of
@@ -89,7 +89,7 @@ namespace {
           KABC::Addressee a = (*it);
           if ( a.isEmpty() ) return AsIcon;
 
-          QString contact = AddresseeView::vCardAsHTML( a, 0L, 0, false, true, true, true, true, true, false );
+          QString contact = AddresseeView::vCardAsHTML( a, 0L, AddresseeView::NoLinks, false, AddresseeView::DefaultFields );
           writer->queue( contact );
 
           QString addToLinkText = i18n( "[Add this contact to the addressbook]" );
@@ -138,17 +138,21 @@ namespace {
   class Plugin : public KMail::Interface::BodyPartFormatterPlugin {
   public:
     const KMail::Interface::BodyPartFormatter * bodyPartFormatter( int idx ) const {
-      return idx == 0 ? new Formatter() : 0 ;
+      return validIndex( idx ) ? new Formatter() : 0 ;
     }
     const char * type( int idx ) const {
-      return idx == 0 ? "text" : 0 ;
+      return validIndex( idx ) ? "text" : 0 ;
     }
     const char * subtype( int idx ) const {
       return idx == 0 ? "x-vcard" : idx == 1 ? "vcard" : 0 ;
     }
 
     const KMail::Interface::BodyPartURLHandler * urlHandler( int idx ) const {
-       return idx == 0 ? new UrlHandler() : 0 ;
+       return validIndex( idx ) ? new UrlHandler() : 0 ;
+    }
+  private:
+    bool validIndex( int idx ) const {
+      return ( idx >= 0 && idx <= 1 );
     }
   };
 
